package org.JUnitTest;



import java.awt.Point;



/**

 * This class implements a simple rectangle class in two-dimensional space. The
 * rectangle coordinates cannot be negative; height and width must be positive. 
 * The following coordinate system is assumed:

 * 
 *          (0,0)- - - - - (5,0)

 *            |

 *            |

 *            |

 *            |

 *           (0,5)

 *

 * @author Alptug Dilek

 */


public class Rectangle

{

// -----------------------------------------------------------------------------

// Section: Instance variables.

// -----------------------------------------------------------------------------

	private double height;

	private double width;

	private Point leftTop;



// -----------------------------------------------------------------------------

// Section: Constructors and initialization.

// -----------------------------------------------------------------------------

	/**

	 * This constructor creates a new rectangle. The last line of the

	 * constructor makes sure that the rectangle is consistent.

	 */

	public Rectangle(Point leftTopPoint,

		double width,

		double height)

	{

		assert leftTopPoint != null;

		this.leftTop = new Point(leftTopPoint);

		this.width = width;

		this.height = height;

		assert this.check();

	}


// -----------------------------------------------------------------------------

// Section: Accessor and set methods.

// -----------------------------------------------------------------------------

	/**

	 * Setters perform assertion checks before they return.

	 */

	public double getHeight()

	{

		return height;

	}



	public void setHeight(double height)

	{

		this.height = height;

		assert this.check();

	}



	public double getWidth()

	{

		return width;

	}



	public void setWidth(double width)

	{

		this.width = width;

		assert this.check();

	}



	public Point getLeftTop()

	{

		return leftTop;

	}



	public void setLeftTop(Point leftTop)

	{

		this.leftTop = leftTop;

		assert this.check();

	}



	/**

	 * This method calculates and returns the center point, assertion 
	 * check is performed.

	 * @return center of the rectangle

	 */

	public Point getCenter()

	{

		Point center = new Point();

		center.setLocation(this.leftTop.getX() + width / 2,

			this.leftTop.getY() + height / 2);

		assert center.getX() > this.leftTop.getX() &&

			center.getY() > this.leftTop.getY();



		return center;

	}


	/**

	 * This method that performs translation of the rectangle.

	 * @param deltaX

	 * @param deltaY

	 */

	public void translate (double deltaX, double deltaY)

	{

		this.leftTop.setLocation(this.leftTop.getX() + deltaX,

			this.leftTop.getY() +deltaY);



		assert this.check();

	}



	public double getRight()

	{

		return getLeft() + width / 2;

	}



	public double getBottom()

	{

		return getTop() + getHeight() / 2;

	}



	public double getLeft()

	{

		return this.leftTop.getX();

	}



	public double getTop()

	{

		return this.leftTop.getY();

	}



// -----------------------------------------------------------------------------

// Section: Debugging methods.

// -----------------------------------------------------------------------------

	/**

	 * This method returns true if the user does not violate the invariants
	 * of this class. This method is used for assertion chekcs in most 
	 * public methods and constructor.

	 * @return boolean

	 */

	private boolean check()

	{

		boolean isLeftTopProper = this.leftTop != null;

		boolean isHeightNonNegative = this.height >= 0;

		boolean isWidthNonNegative = this.width >= 0;

		boolean isLeftNonNegative =  false;

		boolean isTopNonNegative =  false;

		boolean isRightGreaterOrEqualThanLeft = false;

		boolean isBottomGreaterOrEqualThanTop = false;



		if (!isLeftTopProper)

		{

			 System.out.println("Left-top point should not be null");

		}



		else

		{

			isLeftNonNegative = getLeft() >= 0;

			isTopNonNegative = getTop() >= 0;



			if (!isLeftNonNegative)

			{

				System.out.println("Left coordinate cannot be negative");

			}



			else

			{

				isRightGreaterOrEqualThanLeft = getRight() >= getLeft();



				if (!isRightGreaterOrEqualThanLeft)

				{

					System.out.println("Right cannot be smaller than left");

				}

			}



			if (!isTopNonNegative)

			{

				System.out.println("Top coordinate cannot be negative");

			}



			else

			{

				isBottomGreaterOrEqualThanTop = getBottom() > getTop();



				if (!isBottomGreaterOrEqualThanTop)

				{

					System.out.println("Bottom cannot be smaller than top");

				}

			}

		}



		if (!isHeightNonNegative)

		{

			System.out.println("Height cannot be negative");

		}



		if (!isWidthNonNegative)

		{

			System.out.println("Width cannot be negative");

		}



		boolean isRectangleConsistent = isHeightNonNegative &&

			isWidthNonNegative && isBottomGreaterOrEqualThanTop &&

			isRightGreaterOrEqualThanLeft;



		return isRectangleConsistent;

	}



// -----------------------------------------------------------------------------

// Section: Remaining methods.

// -----------------------------------------------------------------------------

	public static void main (String [] args)

	{

		System.out.println("Test");

	}

}

