package org.JUnitTest;


import java.awt.Point;

import junit.framework.*;



/**

 * This class tests the org.JUnitTest.Rectangle class using JUnit library.

 *

 * @author Alptug Dilek

 */



public class RectangleTest extends TestCase

{

// -----------------------------------------------------------------------------

// Section: Instance variables.

// -----------------------------------------------------------------------------
	// org.JUnitTest.Rectangle instance on which various tests will be 
	// performed

	private Rectangle rect;



// -----------------------------------------------------------------------------
// Section: Constructors and initialization.

// -----------------------------------------------------------------------------
	// Needed to use dynamic test method determination in a test suite

	public RectangleTest(String str)

	{

		super(str);

	}



	protected void setUp()

	{

		Point p = new Point();

		p.setLocation(topX, topY);

		this.rect = new Rectangle(p, width, height);

	}


// -----------------------------------------------------------------------------
// Section: Remaining Methods.

// -----------------------------------------------------------------------------
	protected void tearDown()

	{

		this.rect = null;

	}



// -----------------------------------------------------------------------------
// Section: Test Methods.

// -----------------------------------------------------------------------------

	/**

	 * This method tests the new location of the top left point so that the 
	 * coordinate values are greater than 0.

	 */

	public void testInitial()

	{

		assertTrue("Left-top point cannot be null",

			this.rect.getLeftTop() != null);



		assertTrue("Coordinate values must be non-negative",

			this.rect.getLeft() >= 0 && this.rect.getTop() >= 0 &&

			this.rect.getBottom() >= this.rect.getTop() &&

			this.rect.getRight() >= this.rect.getLeft());



		assertTrue("Width and height must be positive",

			this.rect.getHeight() > 0 && this.rect.getWidth() > 0);

	}



	/**

	 * This method tests the translation, so that the top left coordinates
	 * are still non-negative.

	 */

	public void testTranslate()

	{

		this.rect.translate(deltaX,deltaY);



		assertTrue("Coordinates cannot be negative",

			this.rect.getLeft() >= 0 && this.rect.getTop() >= 0);

	}



	/**

	 * This method tests the values of the center point.

	 */

	public void testCenter1()

	{

		Point center = this.rect.getCenter();



		assertNotNull("Center point must not be null", center);

		assertEquals(center.getX(),expectedCenterX);

		assertEquals(center.getY(),expectedCenterY);

	}



	/**

	 * This method tests if the values of the center point are greater than 
	 * the top left point.

	 */

	public void testCenter2()

	{

		Point center = this.rect.getCenter();



		assertNotNull("Center point must not be null", center);

		assertTrue("Center x must be gretaer than left top's x",

			center.getX() > this.rect.getLeft());

		assertTrue("Center y must be gretaer than left top's y",

			center.getY() > this.rect.getTop());

	}



	/**

	 * This method tests the new location of the top left point so that the 
	 * coordinate values are greater than 0.

	 */

	public void testNewLeftTop()

	{

		Point newLeftTop = new Point(newX, newY);

		this.rect.setLeftTop(newLeftTop);


		assertTrue("Coordinate values must be non-negative",

			this.rect.getLeft() >= 0 && this.rect.getTop() >= 0);

	}



	/**

	 * This method tests the new width and height of the rectangle so that 
	 * their values are greater than 0.

	 */

	public void testDimension()

	{

		this.rect.setHeight(newHeight);

		this.rect.setWidth(newWidth);


		assertTrue("Width and height cannot be negative",

			this.rect.getWidth() > 0 && this.rect.getHeight() > 0);

	}



// -----------------------------------------------------------------------------
// Section: Class variables.

// -----------------------------------------------------------------------------
	// initial top left locations of the rectangle

	public final static int topX = 3;

	public final static int topY = 5;

	// the translation values

	public final static int deltaX = -2;

	public final static int deltaY = -1;

	// inital height and width of the rectangle

	public final static int height = 6;

	public final static int width = 4;

	// initial expected location of the rectangle

	public final static double expectedCenterX = Math.floor(topX + width/2);

	public final static double expectedCenterY = Math.floor(topY + height/2);

	// new  left top location of the rectangle

	public final static int newX = 10;

	public final static int newY = 1;

	// new  width and height

	public final static int newHeight = 5;

	public final static int newWidth = 20;

}

